#version 430 core

struct PointLight {
	vec3 position;

	float constant;
	float linear;
	float quadratic;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

in vec3 FragPos;
in vec3 Normal;
in vec2 TexCoords;

uniform vec3 viewPos;
uniform vec3 lightPos;

uniform PointLight pointLight;

uniform sampler2D colorTexture;
uniform samplerCube shadowMap;

uniform float far;

out vec4 FragColor;

vec3 CalcPointLight(PointLight light, vec3 normal, vec3 FragPos, vec3 viewDir);
float ShadowCalculation(vec3 fragPos);

vec3 sampleOffsetDirections[20] = vec3[]
(
   vec3( 1,  1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1,  1,  1), 
   vec3( 1,  1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1,  1, -1),
   vec3( 1,  1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1,  1,  0),
   vec3( 1,  0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1,  0, -1),
   vec3( 0,  1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0,  1, -1)
);   

void main()
{	
	vec3 n = normalize(Normal);
	vec3 viewDir = normalize(viewPos - FragPos);

	//vec3 result = CalcDirLight(dirLight, n, viewDir);

	vec3 result = CalcPointLight(pointLight, n, FragPos, viewDir);

	//result = pow(result, vec3(1.0/2.2));

	FragColor = vec4(result, 1.0);
}

vec3 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir) {
	vec3 lightDir = normalize(light.position - fragPos);

	float diff = max(dot(lightDir, normal), 0.0);

	vec3 halfwayDir = normalize(lightDir + viewDir);
	float spec = pow(max(dot(normal, halfwayDir), 0.0), 20.0);

	float distance = length(light.position - fragPos);
	float attentuation = 1.0 / (light.constant + light.linear + light.quadratic * (distance * distance));

	vec3 ambient = light.ambient;
	vec3 diffuse = light.diffuse * diff;
	vec3 specular = light.specular * spec;

	vec3 tex = vec3(texture(colorTexture, TexCoords)).rgb;

	bool tex_on = (tex.r > 0 || tex.g > 0 || tex.b > 0);

	float shadow = ShadowCalculation(FragPos);

	vec3 color = ambient + (1.0 - shadow) * (diffuse + specular);
	
	if (tex_on) color *= tex;

	return color;
}

float ShadowCalculation(vec3 fragPos) {
	vec3 fragToLight = fragPos - lightPos;

	float currentDepth = length(fragToLight);

	float bias = 0.15;
	int samples = 20;
	float viewDistance = length(viewPos - fragPos);
	float diskRadius = (1.0 + (viewDistance / far)) / 25.0;
	float shadow = 0.0;
	float pcfDepth = 0.0;

	for (int i = 0; i < samples;i++) {
		pcfDepth = texture(shadowMap, fragToLight + sampleOffsetDirections[i] * diskRadius).r;
		pcfDepth *= far;
		shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;
	}

	//FragColor = vec4(vec3(pcfDepth / far), 1.0); 

	shadow /= float(samples);
	return shadow;
}